import click


class ConfigGroup(click.Group):
    """
    Custom config-aware command Group to allow executing the same subcommand
    multiple times with different configurations.
    """
    def invoke(self, ctx):
        click.Command.invoke(self, ctx)
        cmd_name, cmd, args = self.resolve_command(ctx, ctx.args)
        confs = ctx.default_map.get(cmd_name, {})
        if isinstance(confs, list):
            for conf in confs:
                ctx.default_map[cmd_name] = conf
                last_result = super(ConfigGroup, self).invoke(ctx)
            return last_result
        else:
            return super(ConfigGroup, self).invoke(ctx)

    def list_commands(self, ctx):
        commands = super(ConfigGroup, self).list_commands(ctx)
        commands = [cmd for cmd in commands if not cmd.startswith('_')]
        return commands


def config_group(name=None, **attrs):
    attrs.setdefault('cls', ConfigGroup)
    return click.command(name, **attrs)



import os
from setuptools import setup

LONG_DESCRIPTION = """
The wq command line tool.
"""


def readme():
    try:
        readme = open('README.md')
    except IOError:
        return LONG_DESCRIPTION
    return readme.read()


def create_wq_namespace():
    """
    Generate the wq namespace package
    (not checked in, as it technically is the parent of this folder)
    """
    if os.path.isdir("wq"):
        return
    os.makedirs("wq")
    init = open(os.path.join("wq", "__init__.py"), 'w')
    init.write("__import__('pkg_resources').declare_namespace(__name__)")
    init.close()


create_wq_namespace()

setup(
    name='wq.core',
    use_scm_version=True,
    author='S. Andrew Sheppard',
    author_email='andrew@wq.io',
    url='https://wq.io/',
    license='MIT',
    packages=['wq', 'wq.core'],
    package_dir={
        'wq.core': '.',
    },
    install_requires=[
        'click<6',
        'PyYAML',
    ],
    setup_requires=[
        'setuptools_scm',
    ],
    namespace_packages=['wq'],
    entry_points='''
       [console_scripts]
       wq=wq.core:wq
       [wq]
       wq.core=wq.core.info
    ''',
    description=LONG_DESCRIPTION.strip(),
    long_description=readme(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: JavaScript',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Build Tools',
    ]
)



from __future__ import print_function
from wq.core import wq
import click


@wq.command()
@click.option("--output", help="Output filename")
@click.argument(
    "libraries",
    nargs=-1,
)
def versions(output, libraries):
    """
    (DEPRECATED) List installed dependencies.  This is a wrapper for
    `pip freeze` and will be removed in wq.core 2.0.
    """
    click.echo(
        "Warning: wq versions is now an alias for pip freeze",
        err=True,
    )
    print_versions(output, libraries)


def print_versions(output, libraries=[]):
    try:
        from pip._internal.operations import freeze
    except ImportError:
        from pip.operations import freeze

    if libraries:
        click.echo(
            "Warning: The libraries argument has no effect",
            err=True,
        )

    deps = freeze.freeze()

    if output:
        with open(output, 'w') as f:
            for dep in deps:
                print(dep, file=f)
    else:
        for dep in deps:
            click.echo(dep)


DOC_LAYOUT = """---
order: {order}
---

wq {name}
{title_line}

wq {name}: {short_help}
Provided by [wq.{mod}](https://wq.io/wq.{mod}).

```shell
$ wq {name} --help

{help}
```"""

INDEX_LAYOUT = """---
order: 0
---

wq
==

The wq command line interface provides a number of utilities for creating and
deploying applications with the wq framework.

```shell
$ wq --help

{help}
```

## Commands

Command | Module | Description
--------|--------|-------------"""
INDEX_ROW = (
    "[{name}](https://wq.io/docs/wq-{name})"
    " | [wq.{mod}](https://wq.io/wq.{mod})"
    " | {short_help}"
)


@wq.command()
@click.pass_context
def _make_docs(ctx):
    command_list = []
    for i, (name, cmd) in enumerate(sorted(wq.commands.items())):
        if name.startswith('_'):
            continue

        with open('wq-%s.md' % name, 'w') as f:
            module_path = cmd.callback.__module__.split('.')
            if module_path[0] != 'wq':
                continue

            cctx = click.Context(cmd, info_name=name, parent=ctx.parent)
            command_info = dict(
                name=name,
                order=i+1,
                title_line='=' * (len(name) + 3),
                short_help=cmd.short_help,
                mod=module_path[1],
                help=cmd.get_help(cctx),
            )
            print(DOC_LAYOUT.format(**command_info), file=f)
            command_list.append(command_info)

    wq_help = wq.get_help(ctx.parent).split('Commands:')[0].strip()
    with open('wq.md', 'w') as f:
        print(INDEX_LAYOUT.format(
           short_help=wq.short_help,
           help=wq_help
        ), file=f)
        for command_info in command_list:
            print(INDEX_ROW.format(**command_info), file=f)
